// Emacs style mode select   -*- C++ -*- 
//-----------------------------------------------------------------------------
//
// $Id:$
//
// Copyright (C) 1993-1996 by id Software, Inc.
//
// This source is available for distribution and/or modification
// only under the terms of the DOOM Source Code License as
// published by id Software. All rights reserved.
//
// The source is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// FITNESS FOR A PARTICULAR PURPOSE. See the DOOM Source Code License
// for more details.
//
// $Log:$
//
// DESCRIPTION:  Heads-up displays
//
//-----------------------------------------------------------------------------

static const char
rcsid[] = "$Id: hu_stuff.c,v 1.4 1997/02/03 16:47:52 b1 Exp $";

#include <ctype.h>

#include "doomdef.h"

#include "z_zone.h"

#include "m_swap.h"

#include "hu_stuff.h"
#include "hu_lib.h"
#include "w_wad.h"

#include "s_sound.h"

#include "doomstat.h"

#include "multires.h"  
#include "p_spec.h" // For HUDS

// Data.
#include "dstrings.h"
#include "sounds.h"

//
// Locally used constants, shortcuts.
//
#define FONTHEIGHT	(SHORT(hu_font[0]->height)) //This is to make things easier
#define HU_TITLE	(mapnames[(gameepisode-1)*9+gamemap-1])
#define HU_TITLE2	(mapnames2[gamemap-1])
#define HU_TITLEP	(mapnamesp[gamemap-1])
#define HU_TITLET	(mapnamest[gamemap-1])
#define HU_TITLEHEIGHT	1
#define HU_TITLEX	0
#define HU_TITLEY	(SCREENHEIGHT - (200-167) - (2 * FONTHEIGHT))
#define HU_TITLEY2	(SCREENHEIGHT - (200-167) - FONTHEIGHT)
#define HU_TITLEY_FS	(SCREENHEIGHT - (2 * FONTHEIGHT)) //FS Are for Full Screen Mode
#define HU_TITLEY2_FS	(SCREENHEIGHT - FONTHEIGHT) 

#define MAXNAMELENGTH	20 //Do We Really Need Anymore

// HUD positions.
#define ST_KEY0WIDTH	8
#define HU_KEY0X	0
#define HU_KEY0X_R	(SCREENWIDTH - ST_KEY0WIDTH)
#define HU_KEY1X	(HU_KEY0X + ST_KEY0WIDTH)
#define HU_KEY1X_R	(HU_KEY0X_R - ST_KEY0WIDTH)
#define HU_KEY2X	(HU_KEY1X + ST_KEY0WIDTH)
#define HU_KEY2X_R	(HU_KEY1X_R - ST_KEY0WIDTH)

typedef struct hu_pos
{
    int posx;  //The X Position for Left side
    int posxr; //The X Position for Right side
    int posy;  //The Y Position
} hu_pos;

typedef struct hu_ammo_stuff
{
    char* name;      //This is not actually Required But I kerp just for a reference.
    char* iconname;
    int   iconheight;
    int   iconwidth;
} hu_ammo_stuff;

hu_ammo_stuff HU_ICON_INFO[]=
{
    { "Bullets", "CLIP", 11, 9},
    { "Shells", "SHELLS", 12, 32},
    { "Cells", "CELLS", 12, 17},
    { "Rockets", "ROCKET", 27, 12},
    { "Stimpack", "STIMPACK", 15, 14},
    { "Armor Green", "ARMOR1", 17, 31},
    { "Armor Blue", "ARMOR2", 17, 31},
    { "Medikit", "MEDIKIT", 15, 28}
};


//This should probably be done with an array
#define HU_HEALTHY_T	20
int HU_HEALTHY;

int HU_ARMORY_T;
int HU_ARMORY;
int HU_BULLETY_T;
int HU_BULLETY;
int HU_SHELLY_T;
int HU_SHELLY;
int HU_CELLY_T;
int HU_CELLY;
int HU_ROCKETY_T;
int HU_ROCKETY;

int HU_KEYY_T;
int HU_KEY0Y;
int HU_KEY1Y;
int HU_KEY2Y;

hu_pos HU_CARD_POS[6];

#define HU_HUDTOGGLE    'h'
#define HU_STATTOGGLE   's'
static int		HU_ACTIVE = 0;
static int		HU_ACTIVES = 0;


#define HU_INPUTTOGGLE	't'
#define HU_INPUTX	HU_MSGX
#define HU_INPUTY	(HU_MSGY + HU_MSGHEIGHT*(SHORT(hu_font[0]->height) +1))
#define HU_INPUTWIDTH	64
#define HU_INPUTHEIGHT	1



char*	chat_macros[] =
{
    HUSTR_CHATMACRO0,
    HUSTR_CHATMACRO1,
    HUSTR_CHATMACRO2,
    HUSTR_CHATMACRO3,
    HUSTR_CHATMACRO4,
    HUSTR_CHATMACRO5,
    HUSTR_CHATMACRO6,
    HUSTR_CHATMACRO7,
    HUSTR_CHATMACRO8,
    HUSTR_CHATMACRO9
};

char*	player_names[] =
{
    HUSTR_PLRGREEN,
    HUSTR_PLRINDIGO,
    HUSTR_PLRBROWN,
    HUSTR_PLRRED
};


char			chat_char; // remove later.
static player_t*	plr;
patch_t*		hu_font[HU_FONTSIZE];
patch_t*		hu_cards[6];
patch_t*		hu_icons[7]; //Ideally this would contains all the hud icons and the Keys as well but this is easier
static hu_textline_t	w_title;
static hu_textline_t	w_title2;
static hu_textline_t	w_hudtext;

boolean			chat_on;
static hu_itext_t	w_chat;
static boolean		always_off = false;
static char		chat_dest[MAXPLAYERS];
static hu_itext_t w_inputbuffer[MAXPLAYERS];

static boolean		message_on;
boolean			message_dontfuckwithme;
static boolean		message_nottobefuckedwith;

static hu_stext_t	w_message;
static int		message_counter;

extern int		showMessages;
extern boolean		automapactive;

static boolean		headsupactive = false;

//
// Builtin map names.
// The actual names can be found in DStrings.h.
//

char*	mapnames[] =	// DOOM shareware/registered/retail (Ultimate) names.
{

    HUSTR_E1M1,
    HUSTR_E1M2,
    HUSTR_E1M3,
    HUSTR_E1M4,
    HUSTR_E1M5,
    HUSTR_E1M6,
    HUSTR_E1M7,
    HUSTR_E1M8,
    HUSTR_E1M9,

    HUSTR_E2M1,
    HUSTR_E2M2,
    HUSTR_E2M3,
    HUSTR_E2M4,
    HUSTR_E2M5,
    HUSTR_E2M6,
    HUSTR_E2M7,
    HUSTR_E2M8,
    HUSTR_E2M9,

    HUSTR_E3M1,
    HUSTR_E3M2,
    HUSTR_E3M3,
    HUSTR_E3M4,
    HUSTR_E3M5,
    HUSTR_E3M6,
    HUSTR_E3M7,
    HUSTR_E3M8,
    HUSTR_E3M9,

    HUSTR_E4M1,
    HUSTR_E4M2,
    HUSTR_E4M3,
    HUSTR_E4M4,
    HUSTR_E4M5,
    HUSTR_E4M6,
    HUSTR_E4M7,
    HUSTR_E4M8,
    HUSTR_E4M9,

    "NEWLEVEL",
    "NEWLEVEL",
    "NEWLEVEL",
    "NEWLEVEL",
    "NEWLEVEL",
    "NEWLEVEL",
    "NEWLEVEL",
    "NEWLEVEL",
    "NEWLEVEL"
};

char*	mapnames2[] =	// DOOM 2 map names.
{
    HUSTR_1,
    HUSTR_2,
    HUSTR_3,
    HUSTR_4,
    HUSTR_5,
    HUSTR_6,
    HUSTR_7,
    HUSTR_8,
    HUSTR_9,
    HUSTR_10,
    HUSTR_11,
	
    HUSTR_12,
    HUSTR_13,
    HUSTR_14,
    HUSTR_15,
    HUSTR_16,
    HUSTR_17,
    HUSTR_18,
    HUSTR_19,
    HUSTR_20,
	
    HUSTR_21,
    HUSTR_22,
    HUSTR_23,
    HUSTR_24,
    HUSTR_25,
    HUSTR_26,
    HUSTR_27,
    HUSTR_28,
    HUSTR_29,
    HUSTR_30,
    HUSTR_31,
    HUSTR_32
};


char*	mapnamesp[] =	// Plutonia WAD map names.
{
    PHUSTR_1,
    PHUSTR_2,
    PHUSTR_3,
    PHUSTR_4,
    PHUSTR_5,
    PHUSTR_6,
    PHUSTR_7,
    PHUSTR_8,
    PHUSTR_9,
    PHUSTR_10,
    PHUSTR_11,
	
    PHUSTR_12,
    PHUSTR_13,
    PHUSTR_14,
    PHUSTR_15,
    PHUSTR_16,
    PHUSTR_17,
    PHUSTR_18,
    PHUSTR_19,
    PHUSTR_20,
	
    PHUSTR_21,
    PHUSTR_22,
    PHUSTR_23,
    PHUSTR_24,
    PHUSTR_25,
    PHUSTR_26,
    PHUSTR_27,
    PHUSTR_28,
    PHUSTR_29,
    PHUSTR_30,
    PHUSTR_31,
    PHUSTR_32
};


char *mapnamest[] =	// TNT WAD map names.
{
    THUSTR_1,
    THUSTR_2,
    THUSTR_3,
    THUSTR_4,
    THUSTR_5,
    THUSTR_6,
    THUSTR_7,
    THUSTR_8,
    THUSTR_9,
    THUSTR_10,
    THUSTR_11,
	
    THUSTR_12,
    THUSTR_13,
    THUSTR_14,
    THUSTR_15,
    THUSTR_16,
    THUSTR_17,
    THUSTR_18,
    THUSTR_19,
    THUSTR_20,
	
    THUSTR_21,
    THUSTR_22,
    THUSTR_23,
    THUSTR_24,
    THUSTR_25,
    THUSTR_26,
    THUSTR_27,
    THUSTR_28,
    THUSTR_29,
    THUSTR_30,
    THUSTR_31,
    THUSTR_32
};


const char*	shiftxform;

const char french_shiftxform[] =
{
    0,
    1, 2, 3, 4, 5, 6, 7, 8, 9, 10,
    11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
    21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
    31,
    ' ', '!', '"', '#', '$', '%', '&',
    '"', // shift-'
    '(', ')', '*', '+',
    '?', // shift-,
    '_', // shift--
    '>', // shift-.
    '?', // shift-/
    '0', // shift-0
    '1', // shift-1
    '2', // shift-2
    '3', // shift-3
    '4', // shift-4
    '5', // shift-5
    '6', // shift-6
    '7', // shift-7
    '8', // shift-8
    '9', // shift-9
    '/',
    '.', // shift-;
    '<',
    '+', // shift-=
    '>', '?', '@',
    'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N',
    'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
    '[', // shift-[
    '!', // shift-backslash - OH MY GOD DOES WATCOM SUCK
    ']', // shift-]
    '"', '_',
    '\'', // shift-`
    'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N',
    'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
    '{', '|', '}', '~', 127

};

const char english_shiftxform[] =
{

    0,
    1, 2, 3, 4, 5, 6, 7, 8, 9, 10,
    11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
    21, 22, 23, 24, 25, 26, 27, 28, 29, 30,
    31,
    ' ', '!', '"', '#', '$', '%', '&',
    '"', // shift-'
    '(', ')', '*', '+',
    '<', // shift-,
    '_', // shift--
    '>', // shift-.
    '?', // shift-/
    ')', // shift-0
    '!', // shift-1
    '@', // shift-2
    '#', // shift-3
    '$', // shift-4
    '%', // shift-5
    '^', // shift-6
    '&', // shift-7
    '*', // shift-8
    '(', // shift-9
    ':',
    ':', // shift-;
    '<',
    '+', // shift-=
    '>', '?', '@',
    'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N',
    'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
    '[', // shift-[
    '!', // shift-backslash - OH MY GOD DOES WATCOM SUCK
    ']', // shift-]
    '"', '_',
    '\'', // shift-`
    'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N',
    'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
    '{', '|', '}', '~', 127
};

char frenchKeyMap[128]=
{
    0,
    1,2,3,4,5,6,7,8,9,10,
    11,12,13,14,15,16,17,18,19,20,
    21,22,23,24,25,26,27,28,29,30,
    31,
    ' ','!','"','#','$','%','&','%','(',')','*','+',';','-',':','!',
    '0','1','2','3','4','5','6','7','8','9',':','M','<','=','>','?',
    '@','Q','B','C','D','E','F','G','H','I','J','K','L',',','N','O',
    'P','A','R','S','T','U','V','Z','X','Y','W','^','\\','$','^','_',
    '@','Q','B','C','D','E','F','G','H','I','J','K','L',',','N','O',
    'P','A','R','S','T','U','V','Z','X','Y','W','^','\\','$','^',127
};

char ForeignTranslation(unsigned char ch)
{
    return ch < 128 ? frenchKeyMap[ch] : ch;
}

void setcard (int x, int xr, int y, int i)
{
HU_CARD_POS[i].posx = x;
HU_CARD_POS[i].posxr = xr;
HU_CARD_POS[i].posy = y;
}

#define FONT_HEIGHT 7
void HU_Init(void)
{

    int		i;
    int		j;
    char	buffer[9];

    if (language==french)
	shiftxform = french_shiftxform;
    else
	shiftxform = english_shiftxform;

HU_HEALTHY =	(HU_HEALTHY_T + (HU_ICON_INFO[4].iconheight - FONT_HEIGHT) / 2);
HU_ARMORY_T =	(HU_HEALTHY_T + HU_ICON_INFO[4].iconheight + 8);
HU_ARMORY =	(HU_ARMORY_T + (HU_ICON_INFO[5].iconheight - FONT_HEIGHT) / 2);
HU_BULLETY_T =	(HU_ARMORY_T + HU_ICON_INFO[5].iconheight + 8);
HU_BULLETY =	(HU_BULLETY_T + (HU_ICON_INFO[0].iconheight - FONT_HEIGHT) / 2);
HU_SHELLY_T =	(HU_BULLETY_T + HU_ICON_INFO[0].iconheight + 8);
HU_SHELLY =	(HU_SHELLY_T + (HU_ICON_INFO[1].iconheight - FONT_HEIGHT) / 2);
HU_CELLY_T =	(HU_SHELLY_T + HU_ICON_INFO[1].iconheight + 8);
HU_CELLY =	(HU_CELLY_T + (HU_ICON_INFO[2].iconheight - FONT_HEIGHT) / 2);
HU_ROCKETY_T =	(HU_CELLY_T + HU_ICON_INFO[2].iconheight + 8);
HU_ROCKETY =	(HU_ROCKETY_T + (HU_ICON_INFO[3].iconheight - FONT_HEIGHT) / 2);

HU_KEYY_T =	(HU_ROCKETY_T + HU_ICON_INFO[3].iconheight + 8);
HU_KEY0Y =	(HU_KEYY_T + FONT_HEIGHT);
HU_KEY1Y =	HU_KEY0Y;
HU_KEY2Y =	HU_KEY0Y;

 setcard (HU_KEY0X, HU_KEY0X_R, HU_KEY0Y, 0);
 setcard (HU_KEY1X, HU_KEY1X_R, HU_KEY1Y, 1);
 setcard (HU_KEY2X, HU_KEY2X_R, HU_KEY1Y, 2);
 setcard (HU_KEY0X, HU_KEY0X_R, HU_KEY0Y, 3);
 setcard (HU_KEY1X, HU_KEY1X_R, HU_KEY1Y, 4);
 setcard (HU_KEY2X, HU_KEY2X_R, HU_KEY2Y, 5);

    // load the heads-up font
    j = HU_FONTSTART;
    for (i=0;i<HU_FONTSIZE;i++)
    {
	sprintf(buffer, "STCFN%.3d", j++);
	hu_font[i] = (patch_t *) W_CacheLumpName(buffer, PU_STATIC);
    }
    for (i=0;i<6;i++)
    {
	sprintf(buffer, "STKEYS%d", i);
	hu_cards[i] = (patch_t *) W_CacheLumpName(buffer, PU_STATIC);
    }

    for (i=0;i<8;i++)
    {
	hu_icons[i] = (patch_t *) W_CacheLumpName(HU_ICON_INFO[i].iconname, PU_STATIC);
    }

}

void HU_Stop(void)
{
    headsupactive = false;
}

void HU_Start(void)
{

    int		i;

    if (headsupactive)
	HU_Stop();

    plr = &players[consoleplayer];
    message_on = false;
    message_dontfuckwithme = false;
    message_nottobefuckedwith = false;
    chat_on = false;

    // create the message widget
    HUlib_initSText(&w_message,
		    HU_MSGX, HU_MSGY, HU_MSGHEIGHT,
		    hu_font,
		    HU_FONTSTART, &message_on);

    //NOTE THE MAPTITLE WIDGIT IS NOW REPLACED BY THE LEVEL STATS HUD WIDGIT IN HU_LevelStat

    // create the chat widget
    HUlib_initIText(&w_chat,
		    HU_INPUTX, HU_INPUTY,
		    hu_font,
		    HU_FONTSTART, &chat_on);

    // create the inputbuffer widgets
    for (i=0 ; i<MAXPLAYERS ; i++)
	HUlib_initIText(&w_inputbuffer[i], 0, 0, 0, 0, &always_off);

    headsupactive = true;

}

extern int setblocks;
void HU_LevelStats(void) //This draws the Level Stats HUD Nothing Particulary Special
{
    int actkills, actsec, actitems, i, f[4];
    char *textline, *s;

    if (setblocks == 11 && !automapactive)
    {
    HUlib_initTextLine(&w_title,
		       HU_TITLEX, HU_TITLEY_FS,
		       hu_font,
		       HU_FONTSTART);

    HUlib_initTextLine(&w_title2,
		       HU_TITLEX, HU_TITLEY2_FS,
		       hu_font,
		       HU_FONTSTART);
    }
    else
    {
    HUlib_initTextLine(&w_title,
		       HU_TITLEX, HU_TITLEY,
		       hu_font,
		       HU_FONTSTART);

    HUlib_initTextLine(&w_title2,
		       HU_TITLEX, HU_TITLEY2,
		       hu_font,
		       HU_FONTSTART);
    }

    switch ( gamemode )
    {
      case shareware:
      case registered:
      case retail:
	s = HU_TITLE;
	break;

      case commercial:
      default:
	 s = HU_TITLE2;
	 break;
    }

    if (!deathmatch)
    {
	actkills = players[0].killcount + players[1].killcount + players[2].killcount + players[3].killcount; 
	actitems = players[0].itemcount + players[1].itemcount + players[2].itemcount + players[3].itemcount; 
	actsec = players[0].secretcount + players[1].secretcount + players[2].secretcount + players[3].secretcount; 

        sprintf (textline, "%s  Kills: %d / %d", s, actkills, totalkills);
	while (*textline) HUlib_addCharToTextLine(&w_title, *(textline++));
	HUlib_drawTextLine(&w_title, false);
	HUlib_eraseTextLine(&w_title);

	sprintf (textline, "Items: %d / %d  Secrets: %d / %d  Time: %d:%.2d", actitems, totalitems, actsec, totalsecret, (leveltime / TICRATE) / 60, (leveltime / TICRATE) % 60); 
	while (*textline) HUlib_addCharToTextLine(&w_title2, *(textline++));
	HUlib_drawTextLine(&w_title2, false);
	HUlib_eraseTextLine(&w_title2);
    }
    else
    {
	for (i = 0; i < 4; i++)
	    f[i] = players[i].frags[0] + players[i].frags[1] + players[i].frags[2] + players[i].frags[3] - 2 * players[i].frags[i];
	if (!levelTimer)
	{	
	    sprintf (textline, "%s%d    %s%d    Time: %d:%.2d", player_names[0], f[0], player_names[1], f[1], (leveltime / TICRATE) / 60, (leveltime / TICRATE) % 60 );
	    while (*textline) HUlib_addCharToTextLine(&w_title, *(textline++));
	    HUlib_drawTextLine(&w_title, false);
	    HUlib_eraseTextLine(&w_title);

	    sprintf (textline, "%s%d    %s%d", player_names[2], f[2], player_names[3], f[3]);
	    while (*textline) HUlib_addCharToTextLine(&w_title2, *(textline++));
	    HUlib_drawTextLine(&w_title2, false);
	    HUlib_eraseTextLine(&w_title2);
	}
	else
	{
	    sprintf (textline, "%s%d    %s%d     Timelimit is enabled", player_names[0], f[0], player_names[1], f[1]);
	    while (*textline) HUlib_addCharToTextLine(&w_title, *(textline++));
	    HUlib_drawTextLine(&w_title, false);
	    HUlib_eraseTextLine(&w_title);

	    sprintf (textline, "%s%d    %s%d     Time Remaining: %d:%.2d", player_names[2], f[2], player_names[3], f[3], ((levelTimeCount+TICRATE) / TICRATE) / 60, ((levelTimeCount+TICRATE) / TICRATE) % 60);
	    while (*textline) HUlib_addCharToTextLine(&w_title2, *(textline++));
	    HUlib_drawTextLine(&w_title2, false);
	    HUlib_eraseTextLine(&w_title2);
	}
    }
}

int mysize (int i)
{
	if (i < 10) return 1;
	if (i < 100) return 2;
	if (i < 1000) return 3;
	return 0;
}

void HU_PACTIVE (void) //This Draws the QuakeWorld V2 Like HUD
{
    char *hudtext;
    int i, kx[6], size;

hu_pos HU_AMMO_POS[] =
{
    { 0, (SCREENWIDTH - HU_ICON_INFO[0].iconwidth), HU_BULLETY},
    { 0, (SCREENWIDTH - HU_ICON_INFO[1].iconwidth), HU_SHELLY},
    { 0, (SCREENWIDTH - HU_ICON_INFO[2].iconwidth), HU_CELLY},
    { 0, (SCREENWIDTH - HU_ICON_INFO[3].iconwidth), HU_ROCKETY},
    { 0, (SCREENWIDTH - HU_ICON_INFO[4].iconwidth), HU_HEALTHY},
    { 0, (SCREENWIDTH - HU_ICON_INFO[5].iconwidth), HU_ARMORY},
    { 0, (SCREENWIDTH - HU_ICON_INFO[6].iconwidth), HU_ARMORY},
    { 0, (SCREENWIDTH - HU_ICON_INFO[7].iconwidth), HU_HEALTHY}

};

hu_pos HU_AMMO_POS_T[] =
{
    { 0, (SCREENWIDTH - HU_ICON_INFO[0].iconwidth), HU_BULLETY_T},
    { 0, (SCREENWIDTH - HU_ICON_INFO[1].iconwidth), HU_SHELLY_T},
    { 0, (SCREENWIDTH - HU_ICON_INFO[2].iconwidth), HU_CELLY_T},
    { 0, (SCREENWIDTH - HU_ICON_INFO[3].iconwidth), HU_ROCKETY_T},
    { 0, (SCREENWIDTH - HU_ICON_INFO[4].iconwidth), HU_HEALTHY_T},
    { 0, (SCREENWIDTH - HU_ICON_INFO[5].iconwidth), HU_ARMORY_T},
    { 0, (SCREENWIDTH - HU_ICON_INFO[6].iconwidth), HU_ARMORY_T},
    { 0, (SCREENWIDTH - HU_ICON_INFO[7].iconwidth), HU_HEALTHY_T}
};

    if (setblocks < 11) return; //Don't Show the HUD Unless it is fullscreen This is most critical
    
    //To make it easier to see your health level i made the icon for < 50 a STIMPACK  hu_icon[4]
    //and for above 50 a MEDIKIT hu_icon[7]
    if (plr->health < 50) i = 4; else i = 7;

    if (plr->cheats&CF_GODMODE)
    {
	sprintf (hudtext, "GOD");
	size = 3;
    }
    else
    {
	sprintf (hudtext, "%i", plr->health);
	size = mysize(plr->health);
    }

    if (HU_ACTIVE == 2)
    {
	V_DrawPatchDirect(HU_AMMO_POS_T[i].posx, HU_AMMO_POS_T[i].posy, FG, hu_icons[i]);
	HUlib_initTextLine(&w_hudtext, HU_AMMO_POS[i].posx, HU_AMMO_POS[i].posy, hu_font, HU_FONTSTART);
    }
    else
    {
	V_DrawPatchDirect(HU_AMMO_POS_T[i].posxr, HU_AMMO_POS_T[i].posy, FG, hu_icons[i]);
	HUlib_initTextLine(&w_hudtext, (SCREENWIDTH - (8 * size)), HU_AMMO_POS[i].posy, hu_font, HU_FONTSTART);
    }


    while (*hudtext)
	HUlib_addCharToTextLine(&w_hudtext, *(hudtext++));

    HUlib_drawTextLine(&w_hudtext, false);
    HUlib_eraseTextLine(&w_hudtext);

    for (i = 0; i < 4; i++) //This makes things shit easy.
	if (plr->ammo[i])
	{
	    if (HU_ACTIVE == 2)
	    {
		V_DrawPatchDirect(HU_AMMO_POS_T[i].posx, HU_AMMO_POS_T[i].posy, FG, hu_icons[i]);
		HUlib_initTextLine(&w_hudtext, HU_AMMO_POS[i].posx, HU_AMMO_POS[i].posy, hu_font, HU_FONTSTART);
	    }
	    else
	    {
		V_DrawPatchDirect(HU_AMMO_POS_T[i].posxr, HU_AMMO_POS_T[i].posy, FG, hu_icons[i]);
		HUlib_initTextLine(&w_hudtext, (SCREENWIDTH - (8 * mysize(plr->ammo[i]))), HU_AMMO_POS[i].posy, hu_font, HU_FONTSTART);
	    }

	    sprintf (hudtext, "%i", plr->ammo[i]);
	    while (*hudtext)
		HUlib_addCharToTextLine(&w_hudtext, *(hudtext++));
        
	    HUlib_drawTextLine(&w_hudtext, false);
	    HUlib_eraseTextLine(&w_hudtext);
	}
    
    if (plr->armorpoints)
    {
	/* FIXME - Why doesn't using HU_ARMORY_T work? */
	if (plr->armorpoints > 100) i = 6; else i = 5;

	   if (HU_ACTIVE == 2)
	    {
		V_DrawPatchDirect(HU_AMMO_POS_T[i].posx, (HU_ARMORY - 6), FG, hu_icons[i]);
		HUlib_initTextLine(&w_hudtext, HU_AMMO_POS[i].posx, HU_AMMO_POS[i].posy, hu_font, HU_FONTSTART);
	    }
	    else
	    {
		V_DrawPatchDirect(HU_AMMO_POS_T[i].posxr, (HU_ARMORY - 6), FG, hu_icons[i]);
		HUlib_initTextLine(&w_hudtext, (SCREENWIDTH - (8 * mysize(plr->armorpoints))), HU_AMMO_POS[i].posy, hu_font, HU_FONTSTART);
	    }


	sprintf (hudtext, "%i", plr->armorpoints);
	while (*hudtext)
	    HUlib_addCharToTextLine(&w_hudtext, *(hudtext++));
        
	HUlib_drawTextLine(&w_hudtext, false);
	HUlib_eraseTextLine(&w_hudtext);
    }
    

    if (!deathmatch && (plr->cards[0] || plr->cards[1] || plr->cards[2] || plr->cards[3] || plr->cards[4] || plr->cards[5]))
    {
	sprintf (hudtext, "KEYS");

	   if (HU_ACTIVE == 2)
	    {
		HUlib_initTextLine(&w_hudtext, 0, HU_KEYY_T, hu_font, HU_FONTSTART);
		for (i = 0;i < 6;  i++) kx[i] = HU_CARD_POS[i].posx;
	    }
	    else
	    {
		HUlib_initTextLine(&w_hudtext, (SCREENWIDTH - 32), HU_KEYY_T, hu_font, HU_FONTSTART);
		for (i = 0;i < 6;  i++) kx[i] = HU_CARD_POS[i].posxr;
	    }

	while (*hudtext)
	    HUlib_addCharToTextLine(&w_hudtext, *(hudtext++));
        
	HUlib_drawTextLine(&w_hudtext, false);
	HUlib_eraseTextLine(&w_hudtext);
	for (i = 0; i < 6; i++)
	    if (plr->cards[i])
		V_DrawPatchDirect(kx[i], HU_CARD_POS[i].posy, FG, hu_cards[i]);
    }

}

void HU_Drawer(void)
{
    HUlib_drawSText(&w_message);
    HUlib_drawIText(&w_chat);
    if (automapactive || HU_ACTIVES == 1 || plr->playerstate&PST_DEAD) HU_LevelStats();
    if ((HU_ACTIVE != 0) && !automapactive) HU_PACTIVE();


}

void HU_Erase(void)
{

    HUlib_eraseSText(&w_message);
    HUlib_eraseIText(&w_chat);

}

int rscmp (char* s1, char* s2, int n)
{
	int		c1, c2;
	
	do
	{
		c1 = *s1++;
		c2 = *s2++;

		if (!n--)
			return 0;		// strings are equal until end point
		
		if (c1 != c2)
		{
			if (c1 >= 'a' && c1 <= 'z')
				c1 -= ('a' - 'A');
			if (c2 >= 'a' && c2 <= 'z')
				c2 -= ('a' - 'A');
			if (c1 != c2)
				return 1;		// strings not equal
		}
	} while (c1);
	
	return 0;		// strings are equal
}

void HU_Ticker(void)
{

    int i, rc;
    char c, *chat_string;

    // tick down message counter if message is up
    if (message_counter && !--message_counter)
    {
	message_on = false;
	message_nottobefuckedwith = false;
    }

    if (showMessages || message_dontfuckwithme)
    {

	// display message if necessary
	if ((plr->message && !message_nottobefuckedwith)
	    || (plr->message && message_dontfuckwithme))
	{
	    HUlib_addMessageToSText(&w_message, 0, plr->message);
	    plr->message = 0;
	    message_on = true;
	    message_counter = HU_MSGTIMEOUT;
	    message_nottobefuckedwith = message_dontfuckwithme;
	    message_dontfuckwithme = 0;
	}

    } // else message_on = false;

    // check for incoming chat characters
    if (netgame)
    {
	for (i=0 ; i<MAXPLAYERS; i++)
	{
	    if (!playeringame[i])
		continue;
	    if ((c = players[i].cmd.chatchar))
	    {
		if (c <= HU_BROADCAST)
		    chat_dest[i] = c;
		else
		{
		    if (c >= 'a' && c <= 'z')
			c = (char) shiftxform[(unsigned char) c];
		    rc = HUlib_keyInIText(&w_inputbuffer[i], c);
		    if (rc && c == KEYD_ENTER)
		    {
			if (w_inputbuffer[i].l.len
			    && (chat_dest[i] == consoleplayer+1
				|| chat_dest[i] == HU_BROADCAST ))
			{
			    chat_string = w_inputbuffer[i].l.l;

                            if (rscmp(chat_string, "/name ", 6) && rscmp(chat_string, "/NAME ", 6)) //check to see if name command issued
			    { //Nope it wasn't so it must have been a normal message
			    if (i != consoleplayer)
			    {
			    HUlib_addMessageToSText(&w_message,
						    player_names[i],
						    w_inputbuffer[i].l.l);
			    
			    message_nottobefuckedwith = true;
			    message_on = true;
			    message_counter = HU_MSGTIMEOUT;
			    if ( gamemode == commercial )
			      S_StartSound(0, sfx_radio);
			    else
			      S_StartSound(0, sfx_tink);
			    }
			    }
			    else //a name command was issued so change that players name
			    {
				sprintf(player_names[i], "%s: ", &chat_string[6]);
			    
				message_nottobefuckedwith = true;
				message_on = true;
				message_counter = HU_MSGTIMEOUT;
			   }
			}
			HUlib_resetIText(&w_inputbuffer[i]);
		    }
		}
		players[i].cmd.chatchar = 0;
	    }
	}
    }

}

#define QUEUESIZE		128

static char	chatchars[QUEUESIZE];
static int	head = 0;
static int	tail = 0;


void HU_queueChatChar(char c)
{
    if (((head + 1) & (QUEUESIZE-1)) == tail)
    {
	plr->message = HUSTR_MSGU;
    }
    else
    {
	chatchars[head] = c;
	head = (head + 1) & (QUEUESIZE-1);
    }
}

char HU_dequeueChatChar(void)
{
    char c;

    if (head != tail)
    {
	c = chatchars[tail];
	tail = (tail + 1) & (QUEUESIZE-1);
    }
    else
    {
	c = 0;
    }

    return c;
}

boolean HU_Responder(event_t *ev)
{

    static char		lastmessage[HU_MAXLINELENGTH+1];
    char*		macromessage;
    boolean		eatkey = false;
    static boolean	shiftdown = false;
    static boolean	altdown = false;
    unsigned char 	c;
    int			i;
    int			numplayers;
    
    static char		destination_keys[MAXPLAYERS] =
    {
	HUSTR_KEYGREEN,
	HUSTR_KEYINDIGO,
	HUSTR_KEYBROWN,
	HUSTR_KEYRED
    };
    
    static int		num_nobrainers = 0;

    numplayers = 0;
    for (i=0 ; i<MAXPLAYERS ; i++)
	numplayers += playeringame[i];

    if (ev->data1 == KEYD_RSHIFT)
    {
	shiftdown = ev->type == ev_keydown;
	return false;
    }
    else if (ev->data1 == KEYD_RALT || ev->data1 == KEYD_LALT)
    {
	altdown = ev->type == ev_keydown;
	return false;
    }

    if (ev->type != ev_keydown)
	return false;

    if (!chat_on)
    {
	if (ev->data1 == HU_MSGREFRESH)
	{
	    message_on = true;
	    message_counter = HU_MSGTIMEOUT;
	    eatkey = true;
	}
	else if (netgame && ev->data1 == HU_INPUTTOGGLE)
	{
	    eatkey = chat_on = true;
	    HUlib_resetIText(&w_chat);
	    HU_queueChatChar(HU_BROADCAST);
	}
	else if (ev->data1 == HU_HUDTOGGLE)
	{
	    eatkey = true;
            if (HU_ACTIVE == 0) HU_ACTIVE = 2;
	    else if (HU_ACTIVE == 2) HU_ACTIVE = 1;
	    else if (HU_ACTIVE == 1) HU_ACTIVE = 0;
	}
	else if (ev->data1 == HU_STATTOGGLE)
	{
	    eatkey = true;
	    HU_ACTIVES = 1 - HU_ACTIVES;
	}
	else if (netgame && numplayers > 2)
	{
	    for (i=0; i<MAXPLAYERS ; i++)
	    {
		if (ev->data1 == destination_keys[i])
		{
		    if (playeringame[i] && i!=consoleplayer)
		    {
			eatkey = chat_on = true;
			HUlib_resetIText(&w_chat);
			HU_queueChatChar(i+1);
			break;
		    }
		    else if (i == consoleplayer)
		    {
			num_nobrainers++;
			if (num_nobrainers < 3)
			    plr->message = HUSTR_TALKTOSELF1;
			else if (num_nobrainers < 6)
			    plr->message = HUSTR_TALKTOSELF2;
			else if (num_nobrainers < 9)
			    plr->message = HUSTR_TALKTOSELF3;
			else if (num_nobrainers < 32)
			    plr->message = HUSTR_TALKTOSELF4;
			else
			    plr->message = HUSTR_TALKTOSELF5;
		    }
		}
	    }
	}
    }
    else
    {
	c = ev->data1;
	// send a macro
	if (altdown)
	{
	    c = c - '0';
	    if (c > 9)
		return false;
	    // fprintf(stderr, "got here\n");
	    macromessage = chat_macros[c];
	    
	    // kill last message with a '\n'
	    HU_queueChatChar(KEYD_ENTER); // DEBUG!!!
	    
	    // send the macro message
	    while (*macromessage)
		HU_queueChatChar(*macromessage++);
	    HU_queueChatChar(KEYD_ENTER);
	    
	    // leave chat mode and notify that it was sent
	    chat_on = false;
	    strcpy(lastmessage, chat_macros[c]);
	    plr->message = lastmessage;
	    eatkey = true;
	}
	else
	{
	    if (language==french)
		c = ForeignTranslation(c);
	    if (shiftdown || (c >= 'a' && c <= 'z'))
		c = shiftxform[c];
	    eatkey = HUlib_keyInIText(&w_chat, c);
	    if (eatkey)
	    {
		// static unsigned char buf[20]; // DEBUG
		HU_queueChatChar(c);
		
		// sprintf(buf, "KEY: %d => %d", ev->data1, c);
		//      plr->message = buf;
	    }
	    if (c == KEYD_ENTER)
	    {
		chat_on = false;
		if (w_chat.l.len)
		{
		    strcpy(lastmessage, w_chat.l.l);
		    plr->message = lastmessage;
		}
	    }
	    else if (c == KEYD_ESCAPE)
		chat_on = false;
	}
    }

    return eatkey;

}
