// Emacs style mode select   -*- C++ -*-
//-----------------------------------------------------------------------------
//
// $Id:$
//
// Copyright (C) 1993-1996 by id Software, Inc.
//
// This source is available for distribution and/or modification
// only under the terms of the DOOM Source Code License as
// published by id Software. All rights reserved.
//
// The source is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// FITNESS FOR A PARTICULAR PURPOSE. See the DOOM Source Code License
// for more details.
//
// $Log:$
//
// DESCRIPTION:
//	System interface for sound.
//
//-----------------------------------------------------------------------------

static const char
rcsid[] = "$Id: i_unix.c,v 1.5 1997/02/03 22:45:10 b1 Exp $";

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#include <math.h>

#include <sys/time.h>
#include <sys/types.h>

#ifndef LINUX
//#include <sys/filio.h>
#endif

#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>

#include <allegro.h>

// Timer stuff. Experimental.
#include <time.h>
#include <signal.h>

#include "z_zone.h"

#include "i_system.h"
#include "i_sound.h"
#include "m_argv.h"
#include "m_misc.h"
#include "w_wad.h"

#include "doomdef.h"
#include <bcd.h>

// A quick hack to establish a protocol between
// synchronous mix buffer updates and asynchronous
// audio writes. Probably redundant with gametic.
static int flag = 0;

// The number of internal mixing channels,
//  the samples calculated for each mixing step,
//  the size of the 16bit, 2 hardware channel (stereo)
//  mixing buffer, and the samplerate of the raw data.


// Needed for calling the actual sound output.
#define SAMPLECOUNT             2048    // This seem be optimum so don't change it!

#define NUM_CHANNELS_MAX	100     // This can be no more than 100.
int NUM_CHANNELS;			// This gets set from ALLEGRO.CFG
#define BUFMUL                  4       // It is 2 for 16bit, and 2 for two channels. - DO NOT CHANGE THIS
#define SAMPLERATE              22050   // Hz - DO NOT CHANGE THIS
#define SAMPLESIZE              2       // 16bit - 8bit is just a hack really but can be set

#define MIXBUFFERSIZE           (SAMPLECOUNT*BUFMUL)

// The actual lengths of all sound effects.
int 		lengths[NUMSFX];

// UNIX hack, to be removed.
#if SNDINTR

// Update all 30 millisecs, approx. 30fps synchronized.
// Linux resolution is allegedly 10 millisecs,
//  scale is microseconds.
#define SOUND_INTERVAL          10000

// Get the interrupt. Set duration in millisecs.
int I_SoundSetTimer( int duration_of_tick );
void I_SoundDelTimer( void );
#else
// None?
#endif

// The actual output device.
int	audio_fd;

// The global mixing buffer.
// Basically, samples from all active internal channels
//  are modifed and added, and stored in the buffer
//  that is submitted to the audio device.
signed short    mixbuffer[MIXBUFFERSIZE];

// The channel step amount...
unsigned int	channelstep[NUM_CHANNELS_MAX];
// ... and a 0.16 bit remainder of last step.
unsigned int	channelstepremainder[NUM_CHANNELS_MAX];


// The channel data pointers, start and end.
unsigned char*	channels[NUM_CHANNELS_MAX];
unsigned char*	channelsend[NUM_CHANNELS_MAX];


// Time/gametic that the channel started playing,
//  used to determine oldest, which automatically
//  has lowest priority.
// In case number of active sounds exceeds
//  available channels.
int		channelstart[NUM_CHANNELS_MAX];

// The sound in channel handles,
//  determined on registration,
//  might be used to unregister/stop/modify,
//  currently unused.
int 		channelhandles[NUM_CHANNELS_MAX];

// SFX id of the playing sound effect.
// Used to catch duplicates (like chainsaw).
int		channelids[NUM_CHANNELS_MAX];			

// Pitch to stepping lookup, unused.
int		steptable[256];

// Volume lookups.
int		vol_lookup[256*128];

// Hardware left and right channel volume lookup.
int*		channelleftvol_lookup[NUM_CHANNELS_MAX];
int*		channelrightvol_lookup[NUM_CHANNELS_MAX];

// Audio streams for Allegro.
AUDIOSTREAM *leftchan;
AUDIOSTREAM *rightchan;

//CD Stuff. The Track Map is in SOUNDS.C
int CD_FLAG;
int current_track;
int cd_repeat;
int cdmap[20];

//this function converts raw 11khz, 8-bit data to a SAMPLE* that allegro uses
//it is need cuz allegro only loads samples from wavs and vocs
SAMPLE *raw2SAMPLE(unsigned char *rawdata, int len, int freq)
  {
  SAMPLE *spl;

  spl=malloc(sizeof(SAMPLE));
  spl->bits = 8;
  spl->freq = freq;
  spl->len = len;
  spl->priority = 255;
  spl->loop_start = 0;
  spl->loop_end = len;
  spl->param = -1;
  spl->data=(void *)rawdata;

  return spl;
  }


//
// This function loads the sound data from the WAD lump,
//  for single sound.
// This now returns a SAMPLE*, not a void*
//
SAMPLE*
getsfx
( char*         sfxname,
  int*          len )
{
    unsigned char*      sfx;
    unsigned char*      paddedsfx;
    int                 i;
    int                 size;
    int                 paddedsize;
    char                name[20];
    int                 sfxlump;

    
    // Get the sound data from the WAD, allocate lump
    //  in zone memory.
    sprintf(name, "ds%s", sfxname);

    // Now, there is a severe problem with the
    //  sound handling, in it is not (yet/anymore)
    //  gamemode aware. That means, sounds from
    //  DOOM II will be requested even with DOOM
    //  shareware.
    // The sound list is wired into sounds.c,
    //  which sets the external variable.
    // I do not do runtime patches to that
    //  variable. Instead, we will use a
    //  default sound for replacement.
    if ( W_CheckNumForName(name) == -1 )
      sfxlump = W_GetNumForName("dspistol");
    else
      sfxlump = W_GetNumForName(name);
    
    size = W_LumpLength( sfxlump );

    // Debug.
    // fprintf( stderr, "." );
    //fprintf( stderr, " -loading  %s (lump %d, %d bytes)\n",
    //	     sfxname, sfxlump, size );
    //fflush( stderr );
    
    sfx = (unsigned char*)W_CacheLumpNum( sfxlump, PU_STATIC );

    // Pads the sound effect out to the mixing buffer size.
    // The original realloc would interfere with zone memory.
    paddedsize = ((size-8 + (SAMPLECOUNT-1)) / SAMPLECOUNT) * SAMPLECOUNT;

    // Allocate from zone memory.
    paddedsfx = (unsigned char*)Z_Malloc( paddedsize+8, PU_STATIC, 0 );
    // ddt: (unsigned char *) realloc(sfx, paddedsize+8);
    // This should interfere with zone memory handling,
    //  which does not kick in in the soundserver.

    // Now copy and pad.
    memcpy(  paddedsfx, sfx, size );
    for (i=size ; i<paddedsize+8 ; i++)
        paddedsfx[i] = 128;

    // Remove the cached lump.
    Z_Free( sfx );
    
    // Preserve padded length.
    *len = paddedsize;


    // Return allocated padded data.
    if (strcmp (sfxname, "dbopn") != 0 && strcmp (sfxname, "dbload") != 0 && strcmp (sfxname, "dbcls") != 0)
	return raw2SAMPLE(paddedsfx + 8,*len, 11025);
    else
	return raw2SAMPLE(paddedsfx + 8,*len, 22050);

}


//
// Retrieve the raw data lump index
//  for a given SFX name.
//
int I_GetSfxLumpNum(sfxinfo_t* sfx)
{
    char namebuf[9];
    sprintf(namebuf, "ds%s", sfx->name);
    return W_GetNumForName(namebuf);
}

SAMPLE*
I_StartSound
( int		id,
  int		vol,
  int		sep,
  int		pitch,
  int		priority )
  {
  int i;

  // UNUSED
  priority = 0;

  //handle chainsaw
  if ( id == sfx_sawup
    || id == sfx_sawidl
	 || id == sfx_sawful
	 || id == sfx_sawhit
	 || id == sfx_stnmov
	 || id == sfx_pistol	 )
    {
    // Loop all channels, check.
    for (i=0;i<NUM_CHANNELS;i++)
      {
      if (voice_check(i)==S_sfx[id].data)
        {
        stop_sample(S_sfx[id].data);
        break;
        }
      }
    }
  pitch=(pitch-128)/8+128;
  play_sample(S_sfx[id].data,vol*16,sep,pitch*1000/128,0);

  // Returns a handle
  return S_sfx[id].data;
  }


void I_StopSound (SAMPLE* handle)
  {
  stop_sample(handle);
  }


int I_SoundIsPlaying(SAMPLE* handle)
  {
  int i;

  for (i=0;i<NUM_CHANNELS;i++)
    {
    if (voice_check(i)==handle)
      return TRUE;
    }
  return FALSE;
  }

void
I_UpdateSoundParams
( SAMPLE*	handle,
  int	vol,
  int	sep,
  int	pitch)
  {
  pitch=(pitch-128)/8+128;
  adjust_sample(handle,vol*16,sep,pitch*1000/128,0);
  }

void I_SetSfxVolume(int volume)
{
  // Identical to DOS.
  // Basically, this should propagate
  //  the menu/config file setting
  //  to the state variable used in
  //  the mixing.
  snd_SfxVolume = volume;
  if ((snd_MusicVolume>=0)&&(snd_MusicVolume<=15))
    set_volume(snd_SfxVolume*16,snd_MusicVolume*16);
}


//  allegro does these two things now
void I_UpdateSound(void) { }
void I_SubmitSound(void) { }

void I_ShutdownSound(void)
{    
  // Wait till all pending sounds are finished.
  int done = 0;
  int i;
  

  // FIXME (below).
  fprintf( stderr, "I_ShutdownSound: NOT finishing pending sounds\n");
  fflush( stderr );
  
  while ( !done )
  {
    for( i=0 ; i<8 && !channels[i] ; i++);
    
    // FIXME. No proper channel output.
    //if (i==8)
    done=1;
  }
#ifdef SNDINTR
  I_SoundDelTimer();
#endif
  
  // Cleaning up -releasing the DSP device.
  //close ( audio_fd );

  stop_audio_stream(leftchan);
  stop_audio_stream(rightchan);
  bcd_stop();
  bcd_close();
  remove_sound();

  // Done.
  return;
}



void
I_InitSound()
{ 
    
  int i, p, j;
  FILE *cdmaps;
  
#ifdef SNDINTR
  fprintf( stderr, "I_SoundSetTimer: %d microsecs\n", SOUND_INTERVAL );
  I_SoundSetTimer( SOUND_INTERVAL );
#endif
    
  // Secure and configure sound device first.
  fprintf( stderr, "I_InitSound: ");
  
//  audio_fd = open("/dev/dsp", O_WRONLY);
//  if (audio_fd<0)
//    fprintf(stderr, "Could not open /dev/dsp\n");

  allegro_init();
  install_timer();

  set_config_file("allegro.cfg");

  reserve_voices(16, get_config_int("sound", "midi_voices", -1)); //Allow for 16 Channel DIG MIDI Polyphony
  
  NUM_CHANNELS = get_config_int("sound", "digi_chans", 8); //This sets the channels for the Internal Mixer

  if (get_config_int("sound", "cd_enabled", 0))
  {
	CD_FLAG = 2;
        if (get_config_int("sound", "cd_only", 0)) CD_FLAG = 1;
  }

  if (NUM_CHANNELS < 0) NUM_CHANNELS = 8; //These Are just checks to make sure nothing stupid is happening
  else if (NUM_CHANNELS > NUM_CHANNELS_MAX) NUM_CHANNELS = NUM_CHANNELS_MAX;
  
  if (install_sound(get_config_int("sound", "digi_card", DIGI_AUTODETECT),
   get_config_int("sound", "midi_card", MIDI_AUTODETECT), NULL) != 0)
  {
      fprintf( stderr, "Failed! Unable Configure Audio Device\n");
  }
  else
/*                     
  i = 11 | (2<<16);                                           
  myioctl(audio_fd, SNDCTL_DSP_SETFRAGMENT, &i);
  myioctl(audio_fd, SNDCTL_DSP_RESET, 0);
  
  i=SAMPLERATE;
  
  myioctl(audio_fd, SNDCTL_DSP_SPEED, &i);
  
  i=1;
  myioctl(audio_fd, SNDCTL_DSP_STEREO, &i);
  
  myioctl(audio_fd, SNDCTL_DSP_GETFMTS, &i);
  
  if (i&=AFMT_S16_LE)    
    myioctl(audio_fd, SNDCTL_DSP_SETFMT, &i);
  else
    fprintf(stderr, "Could not play signed 16 data\n");
*/
  fprintf(stderr, "configured audio device\n" );

  leftchan = play_audio_stream((SAMPLECOUNT),
      (SAMPLESIZE * 8), (SAMPLERATE), 255, 0);

  rightchan = play_audio_stream((SAMPLECOUNT),
      (SAMPLESIZE * 8), (SAMPLERATE), 255, 255);
   if (!rightchan) {
      printf("I_InitSound: Error creating audio stream!\n");
   }
   if (!leftchan) {
      printf("I_InitSound: Error creating audio stream!\n");
   }

  if (CD_FLAG)
  {
  printf("I_InitSound: ");
  i = bcd_open();
  if (!i)
  {	
        printf("MSCDEX not found. No CD Audio.\n");
	CD_FLAG = 0;
  }
  else
        printf("MSCDEX version %i.%i\n", i/100, i%100);
  printf("I_InitSound: ");
  
  p = M_CheckParm ("-remap");
  cdmaps = fopen(myargv[p+1], "r");
  if (!p || !cdmaps)
  {
	printf("CD Mapping File Not Found. Using Default CD Mapping.\n");
        for (i = 0; i < 20; i++) cdmap[i] = i+1;
  }
  else
  {
	printf("Using CD Mapping Info from File %s\n", myargv[p+1]);
	for (i = 0; i < 20; i++)
	{
	fscanf(cdmaps, "%i", &j);
	if (j) cdmap[i] = j;
	else cdmap[i] = i+1;
	j = 0;
        }
  }
  }
	
	

  // Initialize external data (all sounds) at start, keep static.
  fprintf( stderr, "I_InitSound: ");
  
  for (i=1 ; i<NUMSFX ; i++)
  { 
    // Alias? Example is the chaingun sound linked to pistol.
    if (!S_sfx[i].link)
    {
      // Load data from WAD file.
      S_sfx[i].data = getsfx( S_sfx[i].name, &lengths[i] );
    }	
    else
    {
      // Previously loaded already?
      S_sfx[i].data = S_sfx[i].link->data;
      lengths[i] = lengths[(S_sfx[i].link - S_sfx)/sizeof(sfxinfo_t)];
    }
  }

  fprintf( stderr, "pre-cached all sound data\n");
  
  // Now initialize mixbuffer with zero.
  for ( i = 0; i< MIXBUFFERSIZE; i++ )
    mixbuffer[i] = 0;
  
  // Finished initialization.
  fprintf(stderr, "I_InitSound: sound module ready\n");
    
}




//
// MUSIC API.
// All Music Done
// All Dummies now work
//

void I_SetMusicVolume(int volume)
{
  // Internal state variable.
  snd_MusicVolume = volume;
  // Now set volume on output device.
  set_volume(-1, snd_MusicVolume * 16);
}

void I_InitMusic(void)           { }
void I_ShutdownMusic(void)       { }

static int	looping=0;
static int      musicdies=-1; //USELESS NOW

void I_PlaySong(MIDI *handle, int looping, int musicnum)
{
    musicinfo_t*	music;
    music = &S_music[musicnum];

	if ((CD_FLAG == 2 && music->actual == 1 && music->cdtrack != 0) || (CD_FLAG == 1 && music->cdtrack != 0))
	{
	    bcd_play_track(cdmap[music->cdtrack-1]);
	    current_track = cdmap[music->cdtrack-1];
	    cd_repeat = looping;
	    return;
        }
	else
	{
	    play_midi(handle, looping);
	    cd_repeat = 0; //This is just to be sure
	}

}

void I_PauseSong (void)
{
    if (cd_repeat == 1) cd_repeat = 2;
    bcd_pause();
    midi_pause();     
}

void I_ResumeSong (void)
{
    if (cd_repeat == 2) cd_repeat = 1;
    midi_resume();
    bcd_resume();
}

void I_StopSong(void)
{

  cd_repeat = 0;
  bcd_stop();
  stop_midi();
}

void I_CD_Repeater(void)
{
  if (cd_repeat == 1)
	if (!bcd_audio_busy())
		bcd_play_track(current_track);
}

void I_UnRegisterSong(MIDI *handle)
{
  // UNUSED.
  handle = 0;
}

int I_RegisterSong(void* data)
{
  // UNUSED.
  data = NULL;
  
  return 1;
}

// Is the song playing?
int I_QrySongPlaying(int handle)
{
  // UNUSED.
  handle = 0;
  return looping || musicdies > gametic;
}



//
// Experimental stuff.
// A Linux timer interrupt, for asynchronous
//  sound output.
// I ripped this out of the Timer class in
//  our Difference Engine, including a few
//  SUN remains...
//  
#ifdef sun
    typedef     sigset_t        tSigSet;
#else    
    typedef     int             tSigSet;
#endif


// We might use SIGVTALRM and ITIMER_VIRTUAL, if the process
//  time independend timer happens to get lost due to heavy load.
// SIGALRM and ITIMER_REAL doesn't really work well.
// There are issues with profiling as well.
static int /*__itimer_which*/  itimer = ITIMER_REAL;

static int sig = SIGALRM;

// Interrupt handler.
void I_HandleSoundTimer( int ignore )
{

  // Debug.
  //fprintf( stderr, "%c", '+' ); fflush( stderr );


   I_UpdateSound();
   I_SubmitSound();

//   Feed sound device if necesary.
//  if ( flag )
//  {
    // Write it to DSP device.
    // write(audio_fd, mixbuffer, SAMPLECOUNT*BUFMUL);


    // Reset flag counter.
//    flag = 0;
//  }
//  else
//    return;
  
  // UNUSED, but required.
  ignore = 0;
  return;
}

// Get the interrupt. Set duration in millisecs.
int I_SoundSetTimer( int duration_of_tick )
{
  // Needed for gametick clockwork.
  struct itimerval    value;
  struct itimerval    ovalue;
  struct sigaction    act;
  struct sigaction    oact;

  int res;
  
  // This sets to SA_ONESHOT and SA_NOMASK, thus we can not use it.
  //     signal( _sig, handle_SIG_TICK );
  
  // Now we have to change this attribute for repeated calls.
  act.sa_handler = I_HandleSoundTimer;
#ifndef sun    
  //ac	t.sa_mask = _sig;
#endif
//  act.sa_flags = SA_RESTART; - No way this will work as it no longer exists
  
  sigaction( sig, &act, &oact );

  value.it_interval.tv_sec    = 0;
  value.it_interval.tv_usec   = duration_of_tick;
  value.it_value.tv_sec       = 0;
  value.it_value.tv_usec      = duration_of_tick;

  // Error is -1.
  res = setitimer( itimer, &value, &ovalue );

  // Debug.
  if ( res == -1 )
    fprintf( stderr, "I_SoundSetTimer: interrupt n.a.\n");
  
  return res;
}


// Remove the interrupt. Set duration to zero.
void I_SoundDelTimer()
{
  // Debug.
  if ( I_SoundSetTimer( 0 ) == -1)
    fprintf( stderr, "I_SoundDelTimer: failed to remove interrupt. Doh!\n");
}

void maxchanvol(void)
{
voice_set_volume(leftchan->voice, 255);
voice_set_volume(rightchan->voice, 255);
}

void minchanvol(void)
{
voice_set_volume(leftchan->voice, 0);
voice_set_volume(rightchan->voice, 0);
}

	
